part of katana_module;

extension ModuleBuildContextExtensions on BuildContext {
  /// Get the model adapter.
  ModelAdapter? get model {
    return AdapterScope.of(this).modelAdapter;
  }

  /// Get the platform adapter.
  PlatformAdapter? get platform {
    return AdapterScope.of(this).platformAdapter;
  }

  /// Get the module role.
  List<RoleConfig> get roles {
    return RoleScope.of(this).roles;
  }

  /// Get the app module config.
  AppModule? get app {
    return AppScope.of(this).app;
  }
}

extension RoleConfigExtension on RoleConfig {
  /// Searches for permissions to be included.
  bool containsPermission(String permission) {
    return permissions.contains(permission);
  }
}

extension NullableRoleConfigExtension on RoleConfig? {
  /// Searches for permissions to be included.
  bool containsPermission(String permission) {
    if (this == null) {
      return false;
    }
    return this!.permissions.contains(permission);
  }
}

extension IconDataExtension on IconData {
  DynamicMap toMap() {
    return <String, dynamic>{
      "code": codePoint,
      if (fontFamily.isNotEmpty) "family": fontFamily,
      if (fontPackage.isNotEmpty) "package": fontPackage,
    };
  }
}

extension ColorExtension on Color {
  DynamicMap toMap() {
    return <String, dynamic>{
      "code": value,
    };
  }
}

extension DynamicMapExtension on DynamicMap {
  IconData? toIconData() {
    if (isEmpty || !containsKey("code")) {
      return null;
    }
    return IconData(
      get("code", 0),
      fontFamily: get<String?>("family", null),
      fontPackage: get<String?>("package", null),
    );
  }

  Color? toColor() {
    if (isEmpty || !containsKey("code")) {
      return null;
    }
    return Color(get("code", 0));
  }
}
