part of katana_module;

/// Base class for all modules.
///
/// You can enable or disable the module by specifying [enabled].
///
/// The [id] is given to restore the same when the module is saved to the server.
@immutable
abstract class Module {
  const Module({
    this.id,
    this.enabled = true,
  });

  static final List<Module? Function(DynamicMap map)> _registeredModules = [];

  static void _registerModule(Module? Function(DynamicMap map) converter) {
    if (_registeredModules.contains(converter)) {
      return;
    }
    _registeredModules.add(converter);
  }

  /// ID of the module.
  final String? id;

  /// `true` if Module is enabled.
  final bool enabled;

  /// Module Type.
  String get type => runtimeType.toString();

  /// Convert the module information from [DynamicMap].
  Module? fromMap(DynamicMap map);

  /// Convert the module information to [DynamicMap].
  DynamicMap toMap();
}
